# Contributing
## Development Environment
Development of this project is done using the supported Python version most recently released. Note that tests are run against all supported versions of Python; see: [Testing & Coverage](#testing-and-coverage) for additional information.

This project uses [uv](https://docs.astral.sh/uv) to manage dependencies. With your fork cloned to your local machine, you can install the project and its dependencies to create a development environment using:

```bash
$ uv venv
$ uv sync --all-extras --dev
```

A [`pre-commit`](https://pre-commit.com) configuration is also provided to create a pre-commit hook so linting errors aren't committed:

```bash
$ pre-commit install
```

[`mypy`](https://mypy-lang.org/) is also used by this project to provide static type checking. It can be invoked using:

```bash
$ mypy .
```

Note that `mypy` is not included as a pre-commit hook.

## Testing and Coverage
A [pytest](https://docs.pytest.org/en/latest/) suite is provided, with coverage reporting from [`pytest-cov`](https://github.com/pytest-dev/pytest-cov). A [`tox`](https://github.com/tox-dev/tox/) configuration is provided to test across all supported versions of Python. Testing will be skipped locally for Python versions that cannot be found; all supported versions are tested in CI.

```bash
$ tox
```

Details on missing coverage, including in the test suite, is provided in the report to allow the user to generate additional tests for full coverage. Full code coverage is expected for the majority of code contributed to this project. Some exceptions are expected, primarily around code whose functionality relies on either user input or the presence of external drives; these interactions are currently not mocked, though this may change in the future.

## Documentation
### MkDocs
Library documentation is written using [MkDocs](https://www.mkdocs.org) and [mkdocstrings](https://mkdocstrings.github.io/) and contained in the `./docs/` directory. Unless otherwise agreed upon, documentation should be provided for all public interfaces. In progress documentation can be viewed using `mkdocs serve`, and a `tox` tonfiguration is also provided to render the static HTML to the `./site/` directory for previewing if desired:

```bash
$ tox -e mkdocs
```

### `cog`
The project README contains some dynamic content generated by [`cog`](https://cog.readthedocs.io/en/latest/). The included `tox` configuration will also these sections, and you can also invoke manually using:

```bash
$ tox -e cog
```
